#include "TrivialDetector.hh"
#include "G4LogicalVolume.hh"
#include "G4VPhysicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4Material.hh"
#include "G4NistManager.hh"
#include "G4Box.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "globals.hh"

// The user-configurable settings
#include "TrivialSimSettings.hh"

//////////////////////////////////////////////////////////////////////
//
// Here you need to include the appropriate G4 classes for
// a (uniform) magnetic field and the related manager classes.
// Check the G4 user's guide for details!
//
#include "G4UniformMagField.hh"
#include "G4FieldManager.hh"
#include "G4TransportationManager.hh"
//
//////////////////////////////////////////////////////////////////////

// User limits for the step length of e- and e+
#include "G4UserLimits.hh"

TrivialDetector::TrivialDetector()
{
  G4cout << "-- TrivialDetector constructed." << G4endl;
}

TrivialDetector::~TrivialDetector()
{
  G4cout << "-- TrivialDetector deleted." << G4endl;
}


/**
 * This overloaded method is the entry point for defining the
 * simulation setup: geometry, materials, and sensitive detectors.
 * In this trivial case, our detector consists simply of a small
 * target cube.This target is placed into the world volume of the
 * simulation - another, bigger cube.The material of the target
 * is lead (Pb) per default and can be changed via the command-line. 
 */
G4VPhysicalVolume* TrivialDetector::Construct() 
{

  G4cout << "-- TrivialDetetor::Construct() entered." << G4endl;

  // Get hold of the user-configurable settings
  const TrivialSimSettings * settings = TrivialSimSettings::getSettings();

  G4String worldName("World");

  // Here we define a simple box of 2m length on each side.
  // This will be the solid of our world volume.
  G4VSolid * worldSolid = new G4Box(worldName, 1*m, 1.*m, 1.*m);

  // The material of the world volume should be simply Air.
  // We could construct air out of its elements, or we can just
  // get it from the G4 built-in database of materials. We will
  // demonstrate the latter option:
  
  // Fetch the singleton manager of the built-in material database.
  G4NistManager* matManager = G4NistManager::Instance();

  // Get the material by name from the built-in material database:
  G4Material * worldMaterial = matManager->FindOrBuildMaterial("G4_AIR");

  G4LogicalVolume * worldLV = 
    new G4LogicalVolume(worldSolid, //< solid of the logical volume
			worldMaterial, //< material of the logical volume
			worldName); //< null pointer to indicate that 
                                    //  it's not a sensitive detector

  // Artificially limit the steps in the world-volume and any daughter volume
  // (steps < 10cm, total track length of one track < 5m)
  worldLV->SetUserLimits(new G4UserLimits(10*cm, 5.*m));

  // We also assign the pointer to the class-member pointer, so that
  // we can modify the volume in other methods of this class
  worldLV_ = worldLV;

  // The following defines the color of the world for visualization purposes:
  worldLV->SetVisAttributes(new G4VisAttributes(G4Colour::White()));
  
  // Into the world logical-volume (worldLV) we will now place 
  // a small target cube made of a configurable material:
  G4String targetName("Target");

  // Here we fetch the material from our settings-singleton.
  G4Material * targetMaterial = settings->getTargetMaterial();
  G4cout << "-- Target material is " << targetMaterial->GetName() << G4endl;

  // Definition of a the target solid:
  G4VSolid * targetSolid = new G4Box(targetName, 0.1*m, 0.1*m, 0.1*m);

  // Combine solid and material into a logical volume:
  G4LogicalVolume * targetLV = new G4LogicalVolume(targetSolid, 
						   targetMaterial, 
						   targetName);

  // We also assign the pointer to the class-member pointer, so that
  // we can modify the volume in other methods of this class
  targetLV_ = targetLV;

  // The following defines the color of the target for visualization purposes:
  targetLV->SetVisAttributes(new G4VisAttributes(G4Colour::Yellow()));

  // Here we place the target 0.5 meters off the center
  // along the x-axis within the world volume: */
  new G4PVPlacement(0, //< null pointer for an unrotated placement
		    G4ThreeVector(0.5*m, 0., 0.), //< put the target 0.5m along the x-axis
		    targetLV, //< the logical volume to be places = the target cube
		    targetName, //< a name for this placement
		    worldLV, //< the mother volume in which we place the target
		    false, //< ... not used
		    0); //< a copy number for the placement
  

  //////////////////////////////////////////////////////////////////////////

  // Here we call another method of this class to set-up the magnetic field
  
  setMagneticField();

  //////////////////////////////////////////////////////////////////////////
  
  // Return the world volume of the simulation. This is simply an unrotated, 
  // not translated physical-volume, which is NOT placed into a mother 
  // logical-volume. Instead of a pointer to the mother logical-volume 
  // a null-pointer is provided:
  return new G4PVPlacement(0,//< unrotated
			   G4ThreeVector(0., 0., 0.), //< untranslated
			   worldLV, //< logical volume of the world volume
			   worldName, //< name of the world volume
			   0, //< no pointer to a mother volume
			   false, //< no boolean operation
			   0); //< copy number of the world volume
}

/**
 * Defines a uniform magnetic field with user-configurable field values
 * and assigns the field
 *
 *  - either to the entire simulation world
 *  - or to the target cube only
 * 
 * For coding details, please read the
 * <a href="../../../../man/g4ug/UsersGuides/ForApplicationDeveloper/html/Detector/electroMagneticField.html#4.3.2" target="_blank">
 * G4 user's guide</a>
 */
void TrivialDetector::setMagneticField()
{

  // First, we get the user configurable settings which hold the
  // field values we want to use:
  const TrivialSimSettings* const settings = TrivialSimSettings::getSettings();

  // We fetch the vector of field values from the settings:
  G4ThreeVector fieldVector = settings->getMagFieldValues();
  
  // Then we create a uniform magnetic field using the above values:
  G4UniformMagField* magField
    = new G4UniformMagField(G4ThreeVector(fieldVector));
  
  // Now a "field-manager" is needed:
  G4FieldManager* fieldMgr
      = G4TransportationManager::GetTransportationManager()
        ->GetFieldManager();

  // The field manager takes care of the magnetic field,
  // and uses it as the default field for the entire setup:
  fieldMgr->SetDetectorField(magField);

  // In case we only want to have a field inside the target cube:
  // Comment-out the line of code above ..
  // .. and add the field-manager to the logical volume of the target
  //targetLV_->SetFieldManager(fieldMgr, true);

  // Necessary for G4 to be able to track particles in a field:
  fieldMgr->CreateChordFinder(magField);

}
